'''
@author: Sebastian Lapuschkin
@maintainer: Sebastian Lapuschkin
@contact: sebastian.lapuschkin@hhi.fraunhofer.de
@date: 30.11.2016
@version: 1.0
@copyright: Copyright (c) 2016, Sebastian Lapuschkin, Alexander Binder, Gregoire Montavon, Klaus-Robert Mueller
@license : BSD-2-Clause
'''

import modules
import model_io
import data_io

import numpy as np ; na = np.newaxis

#load the mnist data
Xtrain = data_io.read('../data/MNIST/train_images.npy')
Ytrain = data_io.read('../data/MNIST/train_labels.npy')

Xtest = data_io.read('../data/MNIST/test_images.npy')
Ytest = data_io.read('../data/MNIST/test_labels.npy')

#transfer the pixel values from [0 255] to [-1 1]
Xtrain = Xtrain / 127.5 -1
Xtest = Xtest / 127.5 -1

#reshape the vector representations of the mnist data back to image format. extend the image vertically and horizontally by 4 pixels each.
Xtrain = np.reshape(Xtrain,[Xtrain.shape[0],28,28,1])
Xtrain = np.pad(Xtrain,((0,0),(2,2),(2,2),(0,0)), 'constant', constant_values = (-1.,))

Xtest = np.reshape(Xtest,[Xtest.shape[0],28,28,1])
Xtest = np.pad(Xtest,((0,0),(2,2),(2,2),(0,0)), 'constant', constant_values = (-1.,))

#transform numeric class labels to indicator vectors.
I = Ytrain[:,0].astype(int)
Ytrain = np.zeros([Xtrain.shape[0],np.unique(Ytrain).size])
Ytrain[np.arange(Ytrain.shape[0]),I] = 1

I = Ytest[:,0].astype(int)
Ytest = np.zeros([Xtest.shape[0],np.unique(Ytest).size])
Ytest[np.arange(Ytest.shape[0]),I] = 1


#model a network according to LeNet-5 architecture
lenet = modules.Sequential([
                            modules.Convolution(filtersize=(5,5,1,10),stride = (1,1)),\
                            modules.Rect(),\
                            modules.SumPool(pool=(2,2),stride=(2,2)),\
                            modules.Convolution(filtersize=(5,5,10,25),stride = (1,1)),\
                            modules.Rect(),\
                            modules.SumPool(pool=(2,2),stride=(2,2)),\
                            modules.Convolution(filtersize=(4,4,25,100),stride = (1,1)),\
                            modules.Rect(),\
                            modules.SumPool(pool=(2,2),stride=(2,2)),\
                            modules.Convolution(filtersize=(1,1,100,10),stride = (1,1)),\
                            modules.Flatten()
                        ])

#train the network.
lenet.train(   X=Xtrain,\
                Y=Ytrain,\
                Xval=Xtest,\
                Yval=Ytest,\
                iters=10**6,\
                lrate=0.001,\
                batchsize=25)

#save the network
model_io.write(lenet, '../LeNet-5.txt')




#a slight variation to test max pooling layers. this model should train faster.
maxnet = modules.Sequential([
                            modules.Convolution(filtersize=(5,5,1,10),stride = (1,1)),\
                            modules.Rect(),\
                            modules.MaxPool(pool=(2,2),stride=(2,2)),\
                            modules.Convolution(filtersize=(5,5,10,25),stride = (1,1)),\
                            modules.Rect(),\
                            modules.MaxPool(pool=(2,2),stride=(2,2)),\
                            modules.Convolution(filtersize=(4,4,25,100),stride = (1,1)),\
                            modules.Rect(),\
                            modules.MaxPool(pool=(2,2),stride=(2,2)),\
                            modules.Convolution(filtersize=(1,1,100,10),stride = (1,1)),\
                            modules.Flatten(),\
                            modules.SoftMax()
                        ])

#train the network.
maxnet.train(   X=Xtrain,\
                Y=Ytrain,\
                Xval=Xtest,\
                Yval=Ytest,\
                iters=10**6,\
                lrate=0.001,\
                batchsize=25)

#save the network
model_io.write(maxnet, '../LeNet-5-maxpooling.txt')